#include "palsol.h"
#include "iohb.h"
#include <iomanip>

#define dfile "./data/d.data"
#define kfile "./data/k.data"
#define mfile "./data/m.data"
#define MAXN 5000000
#define MAXNNZ 400000000
//
// A QEP OF SIZE 8652, FROM ROTORDYNAMIC ANALYSIS
//

using namespace std;

int main()
{
	int m, n, rk, p, nnzm, nnzd, nnzf,
		nnzk, nnzms, nnzds, nnzks,
		 nnztmp, info;
	int *colptrm, *rowindm,
		  *colptrd, *rowindd,
      *colptrf, *rowindf,
      *colptrk, *rowindk,
      *colptrms,*rowindms,
      *colptrds,*rowindds,
      *colptrks,*rowindks;


	double *matd, *matk, *matm, *matf;  // matrix M, D, K
	double nm, nd, nk, tmpnr, relnorm;  // local scalar
	double gamma, delta;		        // scaling factor

	bool   scale = true;

	COMPLEX  sigma(0.0,1.0E5), 	    // shift sigma
			 e, *ev, *tmpev, 	    // eigenvalue and vector
			 *matks, *matds, *matms;// shifted matrix Ms, Ds, Ks
	int    nev = 200; 		        // number of eigenvalue wanted;
	double tol = 0.0E-14;		    // tolerance for eig computing


	// WORKING SPACE
	matms = new COMPLEX[MAXNNZ];
	matds = new COMPLEX[MAXNNZ];
	matks = new COMPLEX[MAXNNZ];
	tmpev = new COMPLEX[MAXN];

	colptrms= new int[MAXN];
	rowindms= new int[MAXNNZ];
	colptrds= new int[MAXN];
	rowindds= new int[MAXNNZ];
	colptrks= new int[MAXN];
	rowindks= new int[MAXNNZ];

	clock_t t; // Timing Variable

	p = 9; // Pade order

	/*---------------- DATA READING-------------------------------*/
	t = clock();
	// read m-matrix
	readHB_newmat_double(mfile, &m, &n, &nnzm, &colptrm, &rowindm, &matm);
  
	// read d-matrix
	readHB_newmat_double(dfile, &m, &n, &nnzd, &colptrd, &rowindd, &matd);
  
	// read k-matrix
	readHB_newmat_double(kfile, &m, &n, &nnzk, &colptrk, &rowindk, &matk);
 
	t = clock() - t;
	cout<<"Time for data reading: "<<t*1.0/CLOCKS_PER_SEC<<endl;  
	cout<<"\t size of problem: "<<m<<endl;
	cout<<"\t nnz(M): "<<nnzm<<"\n\t nnz(D): "<<nnzd<<"\n\t nnz(K): "<< nnzk<<endl;

	cout<<"\n\t shift sigma="<<sigma
	    <<"\n\t NEV="<<nev<<endl
	    <<"\n\t scalling?"<<scale<<endl;

  
	/*-----------Step 1: DEFINE PAL MATRICES-----------*/

	PALMatrix M(n, n, nnzm, matm, rowindm, colptrm);
	PALMatrix C(n, n, nnzd, matd, rowindd, colptrd);
	PALMatrix K(n, n, nnzk, matk, rowindk, colptrk);

	/*-----------Step 2: PROBLEM SET UP-------------*/ 
	PALSol LowRankQEP(p, sigma, nev, M, C, K);
  
	/*-----------Step 3: SOLVE THE PROBLEM-----------*/ 
	LowRankQEP.EigComp();	// Solve the QEP.

	LowRankQEP.PrintEig(8);	// Output the results.
  
	return 0;
}

