#include "palmat.h"
#include<iostream>

/*
 * definition of PAL matrix class 
 */


PALMatrix ::
PALMatrix(int m, int n, int nnz, double *val, int *rowind, int *colptr):
	  m(m), n(n), nnz(nnz), val(val), rowind(rowind), colptr(colptr)
{
	/*
	    reorder the indices of array such that it
	    begin at cloptr[0] = 0 and rowind[0]=0
	*/
	if (colptr[0]==1){
		for(int j=0; j<nnz; j++){ rowind[j]--;}
		for(int j=0; j<=n; j++){ colptr[j]--;}
	}
}



PALMatrix ::
PALMatrix( const PALMatrix& P){
	m	= 	P.m;
	n	=	P.n;
	nnz	=	P.nnz;
	val	=	P.val;
	rowind	=	P.rowind;
	colptr	=	P.colptr;
}



double PALMatrix :: 
norm1()
{
	double nm=0.0, tmpnr;
	for(int j=0;j<n;j++){
	   tmpnr = 0.0;
	   for(int i=colptr[j]; i<colptr[j+1]; i++){
		   tmpnr = tmpnr + ABS(val[i]);
	  }
	   if(tmpnr>nm) nm = tmpnr;
	}
	return nm;
}



void PALMatrix ::
LowRankDcomp (LRMat& E, LRMat& F, int& rk)
{

	/* 	A) Get the row indices I and column indices J,
	 *  	   such that X(I,J) forms the nonzero part 
	 *  	   the matrix X.
	 */ 

	int 	*ridx	= 	new int[ nnz ];	// Row indices.
	int 	*cidx	= 	new int[ nnz ]; // Column indices.
	int 	nr;				// Number of nonzero rows.
	int 	nc;				// Number of nonzero columns.
	int	t;				// Internal variables.

	//	A.1) Find ridx[nr] = non-repeated elements of rowind[].
	nr	= 	1;
	ridx[0]	=	rowind[0];

	for( int i=1; i<nnz; i++) {
		for( int j=0; j<nr; j++) { 	// check whether rowind[i] appeared
						// in ridx[0:nr-1]
			if( rowind[ i ] != ridx[ j ] ) {
				ridx[ nr ] = rowind[ i ];
				nr++;
				break;
			}
		}
	}

	// 	A.2) Find  cidx[nc] = {i | i = colptr[j] != colptr[j+1] }
	nc 	=	0;
	t 	=	 colptr[0];

	for( int i=1; i<n; i++ ) {
		if ( colptr[ i ] != t ) { // there is non-zero elements in col i
			cidx[ nc ] = i - 1;
			t = colptr[ i ];
			nc++;
		}
	}


	/* 	B) Form a dense matrix B = A(I,J).	*/

	double *mat = new double[ nr * nc ]; 
	for( int i=0; i<nr; i++ ) {
		for( int j=0; j<nc; j++ ) { 	// B(i,j) =A[I(i),J(j)]
			t = cidx[ j ]; 		// I(j)
			mat[ j * nr + i ] = 0;
			for( int k=colptr[ t ]; k<colptr[ t+1 ]; k++ ) {
				if( rowind[ k ] == ridx[ i ] ) {
					mat[ j * nr + i] = val[ k ];
					break;
				}
			}
		}
	}

	/* 	C) Apply SVD to B.			*/
	t = (nr<nc)? nc : nr;		// t = max( nc , nr )
	

	int lwork = 5*t;		// SVD WORKING SPACE
	int info;
	double *MS = new double[t];
	double *MU = new double[nr*nc];
	double *MVT = new double[nr*nc];
	double *work = new double[lwork];

	dgesvd( "S", "S", &nr, &nc, mat, &nr, MS, MU, &nr, MVT, &nc,
                       work, &lwork, &info);

	/* 	D) generate E and F.			*/

	//  Check numerical rank.
	rk = (nr<nc)? nr : nc;
	for( int i=0; i<t; i++ ) {
		if( MS[ i ] < MS[ 0 ]*10E-16) {
			rk = i + 1;	
			break;
		}
	}	

	double tnm;
	int inc = 1;

	t = (nr<nc)? nr : nc;		// t = min( nc , nr )

	for(int i=0; i<rk;i++){
		tnm = sqrt( MS[ i ] );
		dscal(&nr, &tnm, &MU[ i*nr ], &inc);	// Scale column of u
		dscal(&nc, &tnm, &MVT[i], &t);		// Scale row of vt
	}

	//	Construct the low-rank matrix in LRMat format.
	LRMat 	EE( 'N', nr, rk, MU, nr, ridx);
	LRMat 	FF( 'T', rk, nc, MVT, t, cidx);
	E=EE;
	F=FF;

	return;
}




void PALMatrix :: 
MultMv(COMPLEX *v, COMPLEX *w, COMPLEX alpha)
{
	int idx; 	

	for(int i=0;i<n;i++){
		for(int j= colptr[i]; j<colptr[i+1]; j++){
	      		idx = rowind[j];  
			w[idx] = w[idx] + alpha*val[j]*v[i];
		}
	}

}// Matrix Vector product w=M*v



void PALMatrix ::
MultMTv(COMPLEX *v, COMPLEX *w, COMPLEX alpha)
{
	int idx; 	

	for(int i=0;i<n;i++){
		for(int j= colptr[i]; j<colptr[i+1]; j++){
	      		idx = rowind[j];  
			w[i] = w[i] + alpha*v[idx]*val[j];
		}
	}

}// Matrix Vector product w=M^T*v



/* LRMat definition */

void LRMat::
MultMv(COMPLEX *v, COMPLEX *w, COMPLEX alpha)
{
	int 	t;
	COMPLEX *tmp = new COMPLEX[m+n]; // max(m,n) is fine


	if(T=='N'){
		// compute temp=A*v
		for(int i=0; i<m; i++){
			tmp[i] = 0;
			for(int j=0; j<n; j++){
				tmp[i] = tmp[i] + alpha*mat[j*m +i]*v[j];
			}
		}

		for(int i=0; i<m; i++){
			t 	= ind[i];
			w[t] 	= w[t] + tmp[i];
		}
	}
	else{
		for( int i=0; i<n; i++){
			tmp[i] = v[ ind[i] ];
		}
		// compute temp=A*v
		for(int i=0; i<m; i++){
			for(int j=0; j<n; j++){
				w[i] = w[i] + alpha*mat[j*lda +i]*tmp[j];
			}
		}

	}
}
