#ifndef _PALMAT_H
#define _PALMAT_H

#include "paldef.h"
#include"mkl.h"


class LRMat {

private:
	int 	m;		// Number of rows for the matrix.
	int 	n;		// Number of columns for the matrix.
	double	*mat;	// Array of non-zero elements of the matrix.
	int	lda;
	int	*ind;
	char	T;		// E or F

public:
	LRMat(){};		// Default constructor.

	LRMat(char T, int m, int n, double *mat, int lda, int *ind) :
		T(T), m(m), n(n), mat(mat), lda(lda), ind(ind){};
				// Constructor.

	void MultMv(COMPLEX *v, COMPLEX *w, COMPLEX alpha);
				// Matrix Vector product w= w + aplha*M*v
};



class PALMatrix { // PAL sparse matrix, only CSC format is supported now.

public:
	int	m; 		    // number of rows for the matrix.
	int	n; 		    // number of columns for the matrix.
	int	nnz;		// number of nonzeros
	double	*val;   // Array of nonzero elements of matrix.
	int	*rowind; 	// Row indices of the elements in matm.
	int 	*colptr;// Elements indices where each column starts.

	PALMatrix() {}; 		// Default constructor.

	PALMatrix( const PALMatrix& P);	// Copy constructor.

	PALMatrix(int m, int n, int nnz, double *val, int *rowind, int *colptr);
					// Constructor.

	virtual double norm1();		// Evaluating the matrix 1-norm
		
	void LowRankDcomp(LRMat& E, LRMat& F, int& rk);
					// Compute the low rank decomposition.
					// (for low-rank matrix only). 

	virtual void MultMv(COMPLEX *v, COMPLEX *w, COMPLEX alpha);
					// Matrix Vector product w= w + aplha*M*v

	virtual void MultMTv(COMPLEX *v, COMPLEX *w, COMPLEX alpha);
					// Matrix Vector product w= w + alpha*M^T*v
};

#endif
